﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;

namespace Fresnel
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        int Nx, Ny, Np, Ni;
        double WL,X1,X2,Y1,Y2,F,INA,dV;
        double[] Pr,Pi,Px1,Px2,Py1,Py2;
        double[] Ac,As;
        double[,] UR,UI,II;


        public MainWindow()
        {
            InitializeComponent();
            myControl.UC = this;
            myControl.SetUC();
            myControl.SetData();
        }

        private void Button1_Click(object sender, RoutedEventArgs e)
        {
            int i,j,N;
            double Ox, Oy;
            myControl.SetUC();

            if (myControl.BEdit)
            {
                System.Windows.MessageBox.Show("dataGridView1が編集中です！！");
            }
            else
            {

                IN_FM();
                CLS_II();
                CAL_Ac_As();

                N = 5;
                Ni = 0;
                for (i = -N; i <= N; i++)
                {
                    for (j = -N; j <= N; j++)
                    {
                        if (i * i + j * j <= N * N)
                        {
                            Ni = Ni + 1;
                            Ox = i * F * INA/N;
                            Oy = j * F * INA/N;
                            CAL_UR_UI(Ox, Oy);
                            CAL_II();
                        }
                    }
                }
                CAL_AVE();
                SetDataGrid();
            }
        }



        void CAL_AVE()
        {
            int j, k;
            for (j = 0; j <= Nx; j++)
            {
                for (k = 0; k <= Ny; k++)
                {
                    II[j, k] = II[j, k] / Ni;
                }
            }
        }


        void CAL_II()
        {
            int j, k;

            for (j = 0; j <= Nx; j++)
            {
                for (k = 0; k <= Ny; k++)
                {
                    II[j,k] = II[j,k] + UR[j,k] * UR[j,k] + UI[j,k] * UI[j,k];
                }
            }
        }

        void CAL_UR_UI( double Ox, double Oy)
        {
            int i, j, k;
            double X, Y, v1x, v2x, v1y, v2y;
            double Acv, Asv, Acw, Asw;
            double Ar, Ai;
            double dx, dy;



            for (j = 0; j <= Nx; j++)
            {
                for (k = 0; k <= Ny; k++)
                {
                    UR[j,k] = 0;
                    UI[j,k] = 0;
                }
            }

            dx = (X2 - X1) / Nx;
            dy = (Y2 - Y1) / Ny;

            for (i = 0; i < Np; i++)
            {
                for (j = 0; j <= Nx; j++)
                {
                    X = X1 + dx * j;
                    for (k = 0; k <= Ny; k++)
                    {
                        Y = Y1 + dy * k;
                        v1x = Math.Sqrt(2 / (WL * F)) * (Px1[i] - X - Ox);
                        v2x = Math.Sqrt(2 / (WL * F)) * (Px2[i] - X - Ox);
                        Acv = fAc(v1x, v2x);
                        Asv = fAs(v1x, v2x);

                        v1y = Math.Sqrt(2 / (WL * F)) * (Py1[i] - Y - Oy);
                        v2y = Math.Sqrt(2 / (WL * F)) * (Py2[i] - Y - Oy);
                        Acw = fAc(v1y, v2y);
                        Asw = fAs(v1y, v2y);

                        Ar = (Acv * Asw + Asv * Acw) / 2;
                        Ai = (Asv * Asw - Acv * Acw) / 2;


                        UR[j,k] = UR[j,k] + Pr[i] * Ar - Pi[i] * Ai;
                        UI[j,k] = UI[j,k] + Pr[i] * Ai + Pi[i] * Ar;

                    }
                }
            }

        }

        double fAc(double v1, double v2)
        {
            double Acr, Ac1, Ac2, Vv1, Vv2, dv, Mv;
            int Sv1, Sv2, Nv, N;
            N = 1000;

            if (v1 < 0)
            {
                Sv1 = -1;
                Vv1 = Math.Abs(v1);
            }
            else
            {
                Sv1 = 1;
                Vv1 = v1;
            }
            if (v2 < 0)
            {
                Sv2 = -1;
                Vv2 = Math.Abs(v2);
            }
            else
            {
                Sv2 = 1;
                Vv2 = v2;
            }

            dv=dV;
            Nv=(int)(Vv1/dv);
            Mv =  Vv1/dv-Nv;
            if (Nv < N)
            { Ac1 = Sv1 * (Ac[Nv] + ((Ac[Nv + 1]) - (Ac[Nv])) * Mv); }
            else
            { Ac1 = Sv1 * 0.5; }

	
            Nv=(int)(Vv2/dv);
            Mv = Vv2 / dv - Nv;
            if (Nv < N)
            { Ac2 = Sv2 * (Ac[Nv] + ((Ac[Nv + 1]) - (Ac[Nv])) * Mv); }
            else
            { Ac2 = Sv2 * 0.5; }

            Acr = Ac2 - Ac1;
            return Acr;
        }

        double fAs(double v1, double v2)
        {
	double Asr,As1,As2,Vv1,Vv2,dv,Mv;
	int Sv1,Sv2,Nv,N;

	N=1000;

	if ( v1 < 0 ) 
	{
		Sv1=-1;
		Vv1=Math.Abs(v1);
	}
	else
	{ 
		Sv1=1;
		Vv1=v1;
	}

	if ( v2 < 0 ) 
	{
		Sv2=-1;
		Vv2=Math.Abs(v2);
	}
	else
	{ 
		Sv2=1;
		Vv2=v2;
	}

	dv=dV;
	Nv=(int)(Vv1/dv);
    Mv = Vv1 / dv - Nv;
	if(Nv < N)
	{As1=Sv1*(As[Nv]+((As[Nv+1])-(As[Nv]))*Mv);}
	else
	{As1=Sv1*0.5;}

	Nv=(int)(Vv2/dv);
    Mv = Vv2 / dv - Nv;

	if(Nv < N)
	{As2=Sv2*(As[Nv]+((As[Nv+1])-(As[Nv]))*Mv);}
	else
	{As2=Sv2*0.5;}

	Asr=As2-As1;
	return Asr;
}




        void CAL_Ac_As()
        {
            int i, N;
            double h, v1, v2, v3;
            double SAc, SAs, fc1, fc2, fc3, fs1, fs2, fs3;

            dV = 0.02;
            N = 1000;
            h = dV / 2;
            SAc = 0;
            SAs = 0;

            Ac = new double[N + 2];
            As = new double[N + 2];
            Ac[0] = 0;
            As[0] = 0;
            for (i = 0; i <= N; i++)
            {
                v1 = 2 * i * h;
                v2 = v1 + h;
                v3 = v2 + h;
                fc1 = Math.Cos(Math.PI * v1 * v1 / 2);
                fc2 = Math.Cos(Math.PI * v2 * v2 / 2);
                fc3 = Math.Cos(Math.PI * v3 * v3 / 2);
                fs1 = Math.Sin(Math.PI * v1 * v1 / 2);
                fs2 = Math.Sin(Math.PI * v2 * v2 / 2);
                fs3 = Math.Sin(Math.PI * v3 * v3 / 2);
                SAc = SAc + (fc1 + 4 * fc2 + fc3) * h / 3;
                SAs = SAs + (fs1 + 4 * fs2 + fs3) * h / 3;
                Ac[i + 1] = SAc;
                As[i + 1] = SAs;
            }
        }

        void CLS_II()
        {
            int j, k;

            UR = new double[Nx+1, Ny+1];
            UI = new double[Nx+1, Ny+1];
            II = new double[Nx+1, Ny+1];
            for (j = 0; j <= Nx; j++)
            {
                for (k = 0; k <= Ny; k++)
                {
                    II[j,k] = 0;
                }
            }
        }

        void IN_FM()
        {
            int i;
            Np = Convert.ToInt32(TextBox1.Text);
            WL = Convert.ToDouble(TextBox2.Text);
            F = Convert.ToDouble(TextBox3.Text);
            INA= Convert.ToDouble(TextBox4.Text);
            X1 = Convert.ToDouble(TextBox5.Text);
            X2 = Convert.ToDouble(TextBox6.Text);
            Nx = Convert.ToInt32(TextBox7.Text);
            Y1 = Convert.ToDouble(TextBox8.Text);
            Y2 = Convert.ToDouble(TextBox9.Text);
            Ny = Convert.ToInt32(TextBox10.Text);

            Pr = new double[Np]; Pi = new double[Np];
            Px1 = new double[Np]; Px2 = new double[Np];
            Py1 = new double[Np]; Py2 = new double[Np];

            //myControl.dataGridView1.Invalidate();

            for (i = 0; i < Np; i++)
            {
                Pr[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2+i].Cells[1].Value);
                Pi[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2 + i].Cells[2].Value);
                Px1[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2 + i].Cells[3].Value);
                Px2[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2 + i].Cells[4].Value);
                Py1[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2 + i].Cells[5].Value);
                Py2[i] = Convert.ToDouble(myControl.dataGridView1.Rows[2 + i].Cells[6].Value);
            }
        }

        private void ComboBox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (myControl != null)
            {
               // myControl.M1 = ComboBox1.SelectedIndex;
            }
        }

        private void MenuItem2_Click(object sender, RoutedEventArgs e)
        {
            if (myControl.BEdit)
            {
                System.Windows.MessageBox.Show("dataGridView1が編集中です！！");
            }
            else
            {
                if (myControl.saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    using (StreamWriter outfile = new StreamWriter(myControl.saveFileDialog1.FileName))
                    {
                        outfile.Write(Out_Text());
                    }
                }
            }
        }

        String Out_Text()
        {
            int i; int j;
            int N = Convert.ToInt32(TextBox1.Text);

            String OutText = "Fresnel\r\n";
            OutText = OutText + "項目\t記号\t値\t単位\r\n";
            OutText = OutText + "矩形パターン数\tN\t" + TextBox1.Text + "\t行\r\n";
            OutText = OutText + "波長\tWL\t" + TextBox2.Text + "\tμm\r\n";
            OutText = OutText + "ギャップ量\tb\t" + TextBox3.Text + "\tμm\r\n";
            OutText = OutText + "照明NA\tINA\t" + TextBox4.Text + "\trad\r\n";
            OutText = OutText + "スクリーンX1\tX\t" + TextBox5.Text + "\tμm\r\n";
            OutText = OutText + "スクリーンX2\tX2\t" + TextBox6.Text + "\tμm\r\n";
            OutText = OutText + "スクリーンX分割数\tNx\t" + TextBox7.Text + "\t整数\r\n";
            OutText = OutText + "スクリーンY1\tY\t" + TextBox8.Text + "\tμm\r\n";
            OutText = OutText + "スクリーンY2\tY2\t" + TextBox9.Text + "\tμm\r\n";
            OutText = OutText + "スクリーンY分割数\tN\t" + TextBox10.Text + "\t整数\r\n";
            OutText = OutText + "パターンデータ\r\n";

            for (i = 0; i <= 6; i++) { OutText = OutText + myControl.dataGridView1.Columns[i].HeaderText + "\t"; }
            OutText = OutText + "\r\n";

            for (i = 0; i <  N+2; i++)
            {
                for (j = 0; j <= 6; j++)
                {
                    OutText = OutText + myControl.dataGridView1.Rows[ i].Cells[j].Value + "\t";
                }
                OutText = OutText + "\r\n";
            }


            return OutText;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Minimized;
            this.WindowState = WindowState.Normal;
        }

         void SetDataGrid()
        {
            int i,j;
            double X,Y;
            var win = new Window1();


            win.myControl2.dataGridView1.ColumnCount = Nx + 2;
            win.myControl2.dataGridView1.RowCount = Ny + 2;
            for (i = 0; i <= Nx; i++)
            {
                X = X1 + i * (X2 - X1) / Nx;
                win.myControl2.dataGridView1.Rows[1 + i].Cells[0].Value=Convert.ToString(X);
            }
            for (j = 0; j <= Ny; j++)
            {
                Y = Y1 + j * (Y2 - Y1) / Ny;
                win.myControl2.dataGridView1.Rows[0].Cells[1+j].Value = Convert.ToString(Y);
                for (i = 0; i <= Nx; i++)
                {
                    win.myControl2.dataGridView1.Rows[1+j].Cells[1 + i].Value = Convert.ToString(II[i,j]);
                }
            }
            win.Show();
        }

         private void MenuItem3_Click(object sender, RoutedEventArgs e)
         {
             int i; int j;
             String[,] HData = new String[13, 6];

             if (myControl.BEdit)
             {
                 System.Windows.MessageBox.Show("dataGridView1が編集中です！！");
             }
             else
             {

                 if (myControl.openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                 {
                     using (Microsoft.VisualBasic.FileIO.TextFieldParser MyReader = new Microsoft.VisualBasic.FileIO.TextFieldParser(myControl.openFileDialog1.FileName))
                     {
                         MyReader.TextFieldType = Microsoft.VisualBasic.FileIO.FieldType.Delimited;
                         MyReader.SetDelimiters("\t");
                         String[] currentRow;
                         for (i = 0; i < 13; i++)
                         {
                             currentRow = MyReader.ReadFields();
                             if (currentRow != null)
                             {
                                 j = 0;
                                 foreach (String currentField in currentRow)
                                 {
                                     HData[i, j] = currentField;
                                     if (j < 5) { j = j + 1; }
                                 }
                             }
                         }

                         if (HData[0, 0] == "Fresnel")
                         {
                             int N = Convert.ToInt32(HData[2, 2]);

                             String[,] RCData = new String[N + 4, 9];
                             for (i = 0; i < N + 3; i++)
                             {
                                 currentRow = MyReader.ReadFields();
                                 j = 0;
                                 foreach (String currentField in currentRow)
                                 {
                                     RCData[i, j] = currentField;
                                     if (j < 7) { j = j + 1; }
                                 }
                             }

                             TextBox1.Text = HData[2, 2];
                             TextBox2.Text = HData[3, 2];
                             TextBox3.Text = HData[4, 2];
                             TextBox4.Text = HData[5, 2];
                             TextBox5.Text = HData[6, 2];
                             TextBox6.Text = HData[7, 2];
                             TextBox7.Text = HData[8, 2];
                             TextBox8.Text = HData[9, 2];
                             TextBox9.Text = HData[10, 2];
                             TextBox10.Text = HData[11, 2];
                             myControl.SetUC();



                             for (i = 0; i < N; i++)
                             {
                                 for (j = 0; j < 7; j++)
                                 {
                                     myControl.dataGridView1.Rows[2 + i].Cells[j].Value = RCData[i + 3, j];
                                 }
                             }

                         }
                         else
                         {
                             System.Windows.MessageBox.Show("ファイル形式が異なります！！");
                         }
                     }
                 }
             }
         }
    }
}
